from swarms_cloud.api_key_generator import generate_api_key
from swarms_cloud.func_api_wrapper import FuncAPIWrapper
from swarms_cloud.main import agent_api_wrapper
from swarms_cloud.rate_limiter import rate_limiter
from swarms_cloud.sky_api import SkyInterface

__all__ = [
    "agent_api_wrapper",
    "rate_limiter",
    "FuncAPIWrapper",
    "generate_api_key",
    "SkyInterface",
]



import os
from typing import Optional
from supabase import create_client, Client
from dotenv import load_dotenv

load_dotenv()


def supabase_env():
    """Supabase environment variables

    Raises:
        RuntimeError: _description_

    Returns:
        _type_: _description_
    """
    try:
        supabase_env = {
            "supabase_url": os.environ.get("SUPABASE_URL"),
            "supabase_key": os.environ.get("SUPABASE_KEY"),
        }
        return supabase_env
    except Exception as error:
        raise RuntimeError(
            f"Error loading supabase env: {error} upload .env file to root directory with SUPABASE_URL and SUPABASE_KEY"
        )


class SwarmCloudUsageLogger:
    """Swarm Cloud Usage Logger



    Args:
        supabase_url (str): _description_
        supabase_key (str): _description_
        supabase_table (str, optional): _description_. Defaults to "swarm_cloud_usage".

    Methods:
        log_usage: _description_
        check_api_key: _description_

    Example:
    >>> from swarms_cloud.supabase_handler import SwarmCloudUsageLogger
    >>> logger = SwarmCloudUsageLogger(
    ...     supabase_url="https://<your_supabase_url>.supabase.co",
    ...     supabase_key="<your_supabase_key>"
    ... )
    >>> log_in = logger.log_usage(
    ...     user_id="user123",
    ...     api_key="sk-<your_api_key>",
    ...     endpoint="/agent",
    ...     usage_type="get",
    ...     tokens_used=1
    ... )
    >>> check_api_key = logger.check_api_key(
    ...     "sk-<your_api_key>"
    ... )



    """

    def __init__(
        self,
        supabase_url: str,
        supabase_key: str,
        supabase_table: Optional[str] = "swarm_cloud_usage",
        *args,
        **kwargs,
    ):
        self.supabase_url = supabase_url
        self.supabase_key = supabase_key
        self.supabase_table = supabase_table
        self.supabase: Client = create_client(
            supabase_url, supabase_key, *args, **kwargs
        )

    def log_usage(
        self,
        user_id: str,
        api_key: str,
        usage_type: str,
        tokens_used: int,
        *args,
        **kwargs,
    ):
        """Log usage of Swarm Cloud API

        Args:
            user_id (str): _description_
            api_key (str): _description_
            endpoint (str): _description_
            usage_type (str): _description_
            tokens_used (int): _description_

        Returns:
            _type_: _description_
        """
        try:
            data = {
                "user_id": user_id,
                "api_key": api_key,
                "usage_type": usage_type,
                "tokens_used": tokens_used,
            }
            response = self.supabase.table("swarm_cloud_usage").insert(data).execute()
            return response
        except Exception as error:
            print(f"Error logging usage: {error}")
            raise RuntimeError(f"Error logging usage: {error}")

    def check_api_key(self, api_key: str = None):
        """Check if api key is valid

        Args:
            api_key (str, optional): _description_. Defaults to None.

        Returns:
            _type_: _description_

        Example:
        >>> from swarms_cloud.supabase_handler import SwarmCloudUsageLogger
        >>> logger = SwarmCloudUsageLogger(
        ...     supabase_url="https://<your_supabase_url>.supabase.co",
        ...     supabase_key="<your_supabase_key>"
        ... )
        >>>
        """
        try:
            query = (
                self.supabas.table("swarm_cloud_usage")
                .select("*")
                .eq("api_key", api_key)
                .execute()
            )
            return len(query.data) > 0
        except Exception as error:
            raise RuntimeError(f"Error checking api key: {error}")


# # Usage example
# logger = SwarmCloudUsageLogger(
#     supabase_url="https://<your_supabase_url>.supabase.co",
#     supabase_key="<your_supabase_key>",
# )

# log_in = logger.log_usage(
#     user_id="user123",
#     api_key="sk-<your_api_key>",
#     endpoint="/agent",
#     usage_type="get",
#     tokens_used=1,
# )

# check_api_key = logger.check_api_key("sk-<your_api_key>")



import sky


class SkyInterface:
    """

    SkyInterface is a wrapper around the sky Python API. It provides a
    simplified interface for launching, executing, stopping, starting, and
    tearing down clusters.


    """

    def __init__(self):
        self.clusters = {}

    def launch(self, task, cluster_name=None, **kwargs):
        """Launch a task on a cluster

        Args:
            task (_type_): _description_
            cluster_name (_type_, optional): _description_. Defaults to None.

        Returns:
            _type_: _description_
        """
        try:
            job_id, handle = sky.launch(task, cluster_name=cluster_name, **kwargs)
            if handle:
                self.clusters[cluster_name] = handle
            return job_id
        except Exception as error:
            print("Error launching cluster:", error)

    def execute(self, task, cluster_name, **kwargs):
        """Execute a task on a cluster

        Args:
            task (_type_): _description_
            cluster_name (_type_): _description_

        Raises:
            ValueError: _description_

        Returns:
            _type_: _description_
        """
        if cluster_name not in self.clusters:
            raise ValueError("Cluster {} does not exist".format(cluster_name))
        try:
            return sky.exec(task, cluster_name, **kwargs)
        except Exception as e:
            print("Error executing on cluster:", e)

    def stop(self, cluster_name, **kwargs):
        """Stop a cluster

        Args:
            cluster_name (_type_): _description_
        """
        try:
            sky.stop(cluster_name, **kwargs)
        except (ValueError, RuntimeError) as e:
            print("Error stopping cluster:", e)

    def start(self, cluster_name, **kwargs):
        """start a cluster

        Args:
            cluster_name (_type_): _description_
        """
        try:
            sky.start(cluster_name, **kwargs)
        except Exception as e:
            print("Error starting cluster:", e)

    def down(self, cluster_name, **kwargs):
        """Down a cluster

        Args:
            cluster_name (_type_): _description_
        """
        try:
            sky.down(cluster_name, **kwargs)
            if cluster_name in self.clusters:
                del self.clusters[cluster_name]
        except (ValueError, RuntimeError) as e:
            print("Error tearing down cluster:", e)

    def status(self, **kwargs):
        """Save a cluster

        Returns:
            _type_: _description_
        """
        try:
            return sky.status(**kwargs)
        except Exception as e:
            print("Error getting status:", e)

    def autostop(self, cluster_name, **kwargs):
        """Autostop a cluster

        Args:
            cluster_name (_type_): _description_
        """
        try:
            sky.autostop(cluster_name, **kwargs)
        except Exception as e:
            print("Error setting autostop:", e)



from pydantic import BaseModel
from typing import Optional, List, Any, Callable


class AgentParameters(BaseModel):
    """Agent Parameters

    Args:
    temperature (float, optional): _description_. Defaults to None.
    model_name (str, optional): _description_. Defaults to None.
    openai_api_key (str, optional): _description_. Defaults to None.
    id (str, optional): _description_. Defaults to None.
    llm (Any, optional): _description_. Defaults to None.
    template (Optional[str], optional): _description_. Defaults to None.
    max_loops (int, optional): _description_. Defaults to 5.
    stopping_condition (Optional[Callable], optional): _description_. Defaults to None.
    loop_interval (int, optional): _description_. Defaults to 1.
    retry_attempts (int, optional): _description_. Defaults to 3.
    retry_interval (int, optional): _description_. Defaults to 1.
    return_history (bool, optional): _description_. Defaults to False.
    stopping_token (str, optional): _description_. Defaults to None.
    dynamic_loops (Optional[bool], optional): _description_. Defaults to False.
    interactive (bool, optional): _description_. Defaults to False.
    dashboard (bool, optional): _description_. Defaults to False.
    agent_name (str, optional): _description_. Defaults to "Autonomous-Agent-XYZ1B".
    agent_description (str, optional): _description_. Defaults to None.
    system_prompt (str, optional): _description_. Defaults to None.
    tools (List[Any], optional): _description_. Defaults to None.
    dynamic_temperature_enabled (Optional[bool], optional): _description_. Defaults to False.
    sop (Optional[str], optional): _description_. Defaults to None.
    sop_list (Optional[List[str]], optional): _description_. Defaults to None.
    saved_state_path (Optional[str], optional): _description_. Defaults to None.
    autosave (Optional[bool], optional): _description_. Defaults to False.
    context_length (Optional[int], optional): _description_. Defaults to 8192.
    user_name (str, optional): _description_. Defaults to "Human:".
    self_healing_enabled (Optional[bool], optional): _description_. Defaults to False.
    code_interpreter (Optional[bool], optional): _description_. Defaults to False.
    multi_modal (Optional[bool], optional): _description_. Defaults to None.
    pdf_path (Optional[str], optional): _description_. Defaults to None.


    """

    temperature: float = None
    model_name: str = None
    openai_api_key: str = None
    id: str = None
    llm: Any = None
    template: Optional[str] = None
    max_loops = 5
    stopping_condition: Optional[Callable] = None
    loop_interval: int = 1
    retry_attempts: int = 3
    retry_interval: int = 1
    return_history: bool = False
    stopping_token: str = None
    dynamic_loops: Optional[bool] = False
    interactive: bool = False
    dashboard: bool = False
    agent_name: str = "Autonomous-Agent-XYZ1B"
    agent_description: str = None
    system_prompt: str = None
    tools: List[Any] = None
    dynamic_temperature_enabled: Optional[bool] = False
    sop: Optional[str] = None
    sop_list: Optional[List[str]] = None
    saved_state_path: Optional[str] = None
    autosave: Optional[bool] = False
    context_length: Optional[int] = 8192
    user_name: str = "Human:"
    self_healing_enabled: Optional[bool] = False
    code_interpreter: Optional[bool] = False
    multi_modal: Optional[bool] = None
    pdf_path: Optional[str] = None
    list_of_pdf: Optional[str] = None
    tokenizer: Optional[Any] = None
    memory: Optional[Any] = None
    preset_stopping_token: Optional[bool] = False


class AgentInput(BaseModel):
    task: str
    img: str = None
    parameters: AgentParameters



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['swarms_cloud']

package_data = \
{'': ['*']}

install_requires = \
['fastapi', 'skypilot', 'supabase', 'swarms']

setup_kwargs = {
    'name': 'swarms-cloud',
    'version': '0.0.8',
    'description': 'Swarms Cloud - Pytorch',
    'long_description': '[![Multi-Modality](agorabanner.png)](https://discord.gg/qUtxnK2NMf)\n\n# Swarms Cloud\nDeploy your autonomous agents to the cloud with infinite scalability, 99% uptime, and a polymorphic api.\n\n\nBase swarms code -> fastapi code is added to top and bottom -> agent is decorated with expose as api decorator -> file is parsed and then -> added onto yaml for -> skypilot\n\n\n# Install\n`pip install swarms-cloud`\n\n\n# Stack\n- Backend: FastAPI\n- Skypilot for container management\n- Stripe for payment tracking\n- Postresql for database\n\n\n\n# License\nMIT',
    'author': 'Kye Gomez',
    'author_email': 'kye@apac.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kyegomez/swarms-cloud',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import time
import logging
from typing import Callable
from functools import wraps
from fastapi import HTTPException

# Initialize logger
logger = logging.getLogger(__name__)


def rate_limiter(max_requests: int, time_span: int):
    """Rate limiter decorator

    Args:
        max_requests (int): Maximum number of requests allowed in the time span
        time_span (int): Time span in seconds
    """

    def decorator(func: Callable):
        last_called = {}

        @wraps(func)
        async def wrapper(*args, **kwargs):
            identifier = str(args[0].client.host)
            try:
                if identifier in last_called:
                    time_diff = time.time() - last_called[identifier]
                    if time_diff < time_span:
                        logger.warning(f"Rate limit exceeded for IP: {identifier}")
                        raise HTTPException(status_code=429, detail="Too many requests")
                last_called[identifier] = time.time()
                return await func(*args, **kwargs)
            except Exception as e:
                logger.error(f"An error occurred: {str(e)}")
                raise

        return wrapper

    return decorator
